/** @module login */

import $ from 'jquery';

/**
 * Submit login form.
 *
 * @function
 * @param {HTMLFormElement} form
 */
const submit = function( form ) {
	let $form = $( form ),
		$submit = $form.find( '[type=submit]' ),
		username = $form.find( '[name=username]' ).val(),
		password = $form.find( '[name=password]' ).val(),
		security = $form.find( '[name=security]' ).val(),
		remember = $form.find( '[name=remember]' ).is( ':checked' ) ? 1 : 0,
		redirect = $form.find( '[name=redirect_to]' ).val(),
		data,
		ajax;

	username = username.trim();
	password = password.trim();

	if ( ! username && ! password ) {
		return false;
	}

	$submit.attr( 'disabled', true );
	$submit.find( 'img' ).show();

	// Send login request.
	data = { username, password, security, remember };
	ajax = peepso.postJson( 'auth.login', data, function() {} ).ret;

	// Handle login response.
	ajax.always( function( data /* or jqXHR */ ) {
		let response = data;

		// Handle non-200 response code.
		if ( response.responseJSON ) {
			response = response.responseJSON;
		}

		$submit.removeAttr( 'disabled' );
		$submit.find( 'img' ).hide();

		if ( response.success ) {
			if ( redirect && window.location.href !== redirect ) {
				window.location = redirect;
			} else {
				window.location.reload( true );
			}
		} else if ( response.errors ) {
			try {
				let title = ( response.data && response.data.dialog_title ) || '';
				pswindow.hide();
				if ( false === pswindow.acknowledge( response.errors, title ) ) {
					form
						.find( '.errlogin' )
						.html( response.errors[ 0 ] )
						.css( 'display', 'block' );
				}

				// Show pending activation message if needed.
				let codes = ( response.data && response.data.error_code ) || [];
				if ( codes.indexOf( 'pending_approval' ) > -1 ) {
					$( '.ps-js-register-activation' ).show();
				}
			} catch ( e ) {}
		} else {
			// Assume non-JSON response.
			console.error( 'Non-JSON ajax response', response );
		}
	} );

	return false;
};

export default { submit };

$( function() {
	/**
	 * Display the login dialog if a session_timeout is returned and authRequired is set to true
	 *
	 * @param {Event} e
	 * @param {boolean} reload
	 */
	$( window ).on( 'peepso_auth_required', function( e, reload ) {
		$( '.login-area input' ).attr( 'disabled', true );
		// Hide any open pswindows
		if ( pswindow.is_visible ) {
			pswindow.hide();
		}
		// TODO: string needs to be translatable
		pswindow.show( peepsodata.login_dialog_title, peepsodata.login_dialog );
		$( document ).trigger( 'peepso_login_shown' );
		$( '#ps-window' ).one( 'pswindow.hidden', function() {
			$( '.login-area input' ).removeAttr( 'disabled' );
		} );
		if ( reload ) {
			$( 'input[name=redirect_to]' ).val( window.location );
		}
	} );
} );
